/* $Id$

This file is part of libmspcore
Copyright © 2007 Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/
#ifndef WIN32
#include <dlfcn.h>
#include <execinfo.h>
#endif
#include "backtrace.h"
#include "demangle.h"

using namespace std;

namespace Msp {
namespace Debug {

Backtrace Backtrace::create()
{
#ifndef WIN32
	void *addresses[50];
	int count=::backtrace(addresses, 50);

	//char **symbols=backtrace_symbols(addresses, count);

	Backtrace bt;
	Dl_info dli;
	for(int i=0; i<count; ++i)
	{
		StackFrame frame;
		frame.address=addresses[i];
		if(dladdr(addresses[i], &dli))
		{
			frame.file=dli.dli_fname;
			if(dli.dli_sname)
				frame.symbol=demangle(dli.dli_sname);
		}
		else
			frame.file="<unknown>";
		bt.frames.push_back(frame);
	}

	//free(symbols);

	return bt;
#else
	return Backtrace();
#endif
}

ostream &operator<<(ostream &out, const Backtrace &bt)
{
	const Backtrace::FrameSeq &frames=bt.get_frames();
	for(Backtrace::FrameSeq::const_iterator i=frames.begin(); i!=frames.end(); ++i)
		out<<i->address<<" in "<<i->symbol<<" from "<<i->file<<'\n';

	return out;
}

} // namespace Debug
} // namespace Msp
