/*
This file is part of libmspcore     
Copyright © 2006  Mikko Rasa, Mikkosoft Productions
Distributed under the LGPL
*/

#include "timer.h"
#include "utils.h"

using namespace std;

namespace Msp {
namespace Time {

Timer::Timer():
	slots(slot_compare)
{ }

Timer::~Timer()
{
	while(!slots.empty())
	{
		delete slots.top();
		slots.pop();
	}
}

Timer::Slot &Timer::add(const TimeDelta &td)
{
	Slot *s=new Slot(td);
	mutex.lock();
	slots.push(s);
	mutex.unlock();
	sem.signal();
	return *s;
}

Timer::Slot &Timer::add(const TimeStamp &ts)
{
	Slot *s=new Slot(ts);
	mutex.lock();
	slots.push(s);
	mutex.unlock();
	sem.signal();
	return *s;
}

void Timer::tick(bool block)
{
	if(slots.empty())
	{
		if(block)
			sem.wait();
		return;
	}

	mutex.lock();
	Slot *next=slots.top();
	mutex.unlock();

	const TimeStamp &stamp=next->get_timeout();
	const TimeStamp t=now();
	if(stamp<=t || (block && sem.wait(stamp-t)==1))
	{
		slots.pop();
		if(next->signal_timeout.emit() && next->increment())
			slots.push(next);
		else
			delete next;
	}
}

bool Timer::slot_compare(Slot *a, Slot *b)
{
	return *a<*b;
}


Timer::Slot::Slot(const TimeDelta &td):
	interval(td),
	timeout(now()+interval)
{ }

Timer::Slot::Slot(const TimeStamp &ts):
	timeout(ts)
{ }

bool Timer::Slot::increment()
{
	if(!interval)
		return false;
	timeout+=interval;
	return true;
}

bool Timer::Slot::operator<(const Slot &other) const
{
	return timeout<other.timeout;
}

} // namespace Time
} // namespace Msp
